@@ -1,5 +1,5 @@
 /* This file is part of GNU Dico.
-   Copyright (C) 2008, 2010 Sergey Poznyakoff
+   Copyright (C) 2008, 2010, 2012 Sergey Poznyakoff
 
    GNU Dico is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -73,7 +73,7 @@ initial_banner(dico_stream_t str)
 	     (unsigned long) time(NULL),
 	     hostname);
     stream_writez(str, msg_id);
-    dico_stream_write(str, "\r\n", 2);
+    dico_stream_write(str, "\n", 1);
 }
 
 int
@@ -152,7 +152,85 @@ close_databases()
     }
     dico_iterator_destroy(&itr);
 }
+
+/* Default selectors for some commonly used methods. Modules should
+   override these. */
+int
+exact_sel(int cmd, dico_key_t key, const char *dict_word)
+{
+    switch (cmd) {
+    case DICO_SELECT_BEGIN:
+	break;
 
+    case DICO_SELECT_RUN:
+	return utf8_strcasecmp(key->word, (char*)dict_word) == 0;
+
+    case DICO_SELECT_END:
+	break;
+    }
+    return 0;
+}
+
+int
+prefix_sel(int cmd, dico_key_t key, const char *dict_word)
+{
+    size_t wordlen, keylen;
+    
+    switch (cmd) {
+    case DICO_SELECT_BEGIN:
+	key->call_data = (void*) utf8_strlen(key->word);
+	break;
+
+    case DICO_SELECT_RUN:
+	keylen = (size_t)key->call_data;
+	wordlen = utf8_strlen(dict_word);
+	return (wordlen >= keylen &&
+		utf8_strncasecmp((char*)dict_word, key->word, keylen) == 0);
+
+    case DICO_SELECT_END:
+	break;
+    }
+    return 0;
+}
+
+struct suffix {
+    size_t charlen;
+    size_t bytelen;
+};
+
+int
+suffix_sel(int cmd, dico_key_t key, const char *dict_word)
+{
+    size_t wordlen;
+    struct suffix *suf;
+    
+    switch (cmd) {
+    case DICO_SELECT_BEGIN:
+	suf = malloc(sizeof(*suf));
+	if (!suf) {
+	    dico_log(L_ERR, ENOMEM, "stdstrat_suffix");
+	    return -1;
+	}
+	suf->bytelen = utf8_strlen(key->word);
+	suf->charlen = strlen(key->word);
+	key->call_data = suf;
+	break;
+
+    case DICO_SELECT_RUN:
+	suf = (struct suffix*)key->call_data;
+	wordlen = strlen(dict_word);
+	return (wordlen >= suf->bytelen &&
+		utf8_strncasecmp((char*)dict_word + (wordlen - suf->bytelen),
+				 key->word, suf->bytelen) == 0);
+
+    case DICO_SELECT_END:
+	free(key->call_data);
+	break;
+    }
+    return 0;
+}
+
+/* Soundex selector */
 int
 soundex_sel(int cmd, dico_key_t key, const char *dict_word)
 {
@@ -160,7 +238,7 @@ soundex_sel(int cmd, dico_key_t key, const char *dict_word)
 
     switch (cmd) {
     case DICO_SELECT_BEGIN:
-	key->call_data = malloc(5);
+	key->call_data = malloc(DICO_SOUNDEX_SIZE);
 	if (!key->call_data)
 	    return 1;
 	dico_soundex(key->word, key->call_data);
@@ -181,8 +259,9 @@ void
 dicod_init_strategies()
 {
     static struct dico_strategy defstrat[] = {
-	{ "exact", "Match words exactly" },
-	{ "prefix", "Match word prefixes" },
+	{ "exact", "Match words exactly", exact_sel },
+	{ "prefix", "Match word prefixes", prefix_sel },
+	{ "suffix", "Match word suffixes", suffix_sel },
 	{ "soundex", "Match using SOUNDEX algorithm", soundex_sel, NULL },
     };
     int i;
@@ -243,13 +322,11 @@ dicod_loop(dico_stream_t str)
     char *buf = NULL;
     size_t size = 0;
     size_t rdbytes;
-    xdico_input_t input;
-    int argc;
-    char **argv;
+    struct dico_tokbuf tb;
     
     begin_timing("dicod");
     signal(SIGALRM, sig_alarm);
-    memset(&input, 0, sizeof input);
+
     got_quit = 0;
     if (transcript) {
 	dico_stream_t logstr = dico_log_stream_create(L_DEBUG);
@@ -268,15 +345,15 @@ dicod_loop(dico_stream_t str)
     check_db_visibility();
     initial_banner(iostr);
 
-    input = xdico_tokenize_begin();
+    dico_tokenize_begin(&tb);
     while (!got_quit && get_input_line(iostr, &buf, &size, &rdbytes) == 0) {
 	trimnl(buf, rdbytes);
-	xdico_tokenize_input(input, buf, &argc, &argv);
-	if (argc == 0)
+	xdico_tokenize_string(&tb, buf);
+	if (tb.tb_tokc == 0)
 	    continue;
-	dicod_handle_command(iostr, argc, argv);
+	dicod_handle_command(iostr, tb.tb_tokc, tb.tb_tokv);
     }
-    xdico_tokenize_end(&input);
+    dico_tokenize_end(&tb);
     close_databases();    
     init_auth_data();
     access_log_free_cache();
@@ -288,11 +365,9 @@ dicod_loop(dico_stream_t str)
 int
 dicod_inetd()
 {
-    dico_stream_t str = dico_fd_io_stream_create(0, 1);
-
+    dico_stream_t str = dicod_iostream(0, 1);
     if (!str)
-        return 1;
-    dico_stream_set_buffer(str, dico_buffer_line, DICO_MAX_BUFFER);
+	return 1;
 
     client_addrlen = sizeof(client_addr);
     if (getsockname (0, &client_addr, &client_addrlen) == -1)
@@ -304,3 +379,26 @@ dicod_inetd()
     
     return dicod_loop(str);
 }
+
+dico_stream_t
+dicod_iostream(int ifd, int ofd)
+{
+    dico_stream_t str = dico_fd_io_stream_create(ifd, ofd);
+	
+    if (!str)
+        return NULL;
+    dico_stream_set_buffer(str, dico_buffer_line, DICO_MAX_BUFFER);
+    if (!isatty(ifd)) {
+	dico_stream_t s = dico_crlf_stream(str,
+					   DICO_STREAM_READ|DICO_STREAM_WRITE,
+					   0);
+	if (!s) {
+	    dico_stream_destroy(&str);
+	    return NULL;
+	}
+	str = s;
+	dico_stream_set_buffer(str, dico_buffer_line, DICO_MAX_BUFFER);
+    }
+    return str;
+}
+
