@@ -1,5 +1,5 @@
 /* This file is part of GNU Dico.
-   Copyright (C) 2008, 2010 Sergey Poznyakoff
+   Copyright (C) 2008, 2010, 2012 Sergey Poznyakoff
 
    GNU Dico is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -55,22 +55,25 @@ open_sockets()
 {
     size_t i;
     dico_iterator_t itr;
-    sockaddr_union_t *sp;
+    struct grecs_sockaddr *sp;
     struct stat st;
     int t;
-    int socklen;
     char *p;
     
     srvcount = dico_list_count(listen_addr);
     if (srvcount == 0) {
 	/* Provide defaults */
-	struct sockaddr_in *sp = xmalloc(sizeof(*sp));
+	struct sockaddr_in *s_in = xmalloc(sizeof(*s_in));
+
+	sp = xmalloc(sizeof(*sp));
+	sp->sa = (struct sockaddr*) s_in;
+	sp->len = sizeof(*s_in);
 	
 	if (!listen_addr)
 	    listen_addr = xdico_list_create();
-	sp->sin_family = AF_INET;
-	sp->sin_addr.s_addr = INADDR_ANY;
-	sp->sin_port = htons(DICO_DICT_PORT);
+	s_in->sin_family = AF_INET;
+	s_in->sin_addr.s_addr = INADDR_ANY;
+	s_in->sin_port = htons(DICO_DICT_PORT);
 	xdico_list_append(listen_addr, sp);
 	srvcount = 1;
     }
@@ -79,48 +82,53 @@ open_sockets()
     itr = xdico_list_iterator(listen_addr);
     for (i = 0, sp = dico_iterator_first(itr); sp;
 	 sp = dico_iterator_next(itr)) {
-	int fd = socket(address_family_to_domain(sp->s.sa_family),
+	int fd = socket(address_family_to_domain(sp->sa->sa_family),
 			SOCK_STREAM, 0);
 	if (fd == -1) {
 	    dico_log(L_ERR, errno, "socket");
 	    continue;
 	}
 
-	switch (sp->s.sa_family) {
-	case AF_UNIX:
-	    if (stat(sp->s_un.sun_path, &st)) {
+	switch (sp->sa->sa_family) {
+	case AF_UNIX: {
+	    struct sockaddr_un *s_un = (struct sockaddr_un *)sp->sa;
+	    if (stat(s_un->sun_path, &st)) {
 		if (errno != ENOENT) {
 		    dico_log(L_ERR, errno,
 			   _("file %s exists but cannot be stat'd"),
-			   sp->s_un.sun_path);
+			   s_un->sun_path);
 		    close(fd);
 		    continue;
 		}
 	    } else if (!S_ISSOCK(st.st_mode)) {
 		dico_log(L_ERR, 0,
 		       _("file %s is not a socket"),
-		       sp->s_un.sun_path);
+		       s_un->sun_path);
 		close(fd);
 		continue;
-	    } else if (unlink(sp->s_un.sun_path)) {
+	    } else if (unlink(s_un->sun_path)) {
 		dico_log(L_ERR, errno,
-		       _("cannot unlink file %s"),
-		       sp->s_un.sun_path);
+			 _("cannot unlink file %s"),
+			 s_un->sun_path);
 		close(fd);
 		continue;
 	    }
-	    socklen = sizeof(sp->s_un);
 	    break;
-	
+	}
+	    
 	case AF_INET:
 	    t = 1;	 
 	    setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, &t, sizeof(t));
-	    socklen = sizeof(sp->s_in);
 	    break;
+
+	default:
+	    dico_log(L_ERR, 0, _("unsupported address family: %d"),
+		     sp->sa->sa_family);
+	    continue;
 	}
 
-	if (bind(fd, &sp->s, socklen) == -1) {
-	    p = sockaddr_to_astr(&sp->s, socklen);
+	if (bind(fd, sp->sa, sp->len) == -1) {
+	    p = sockaddr_to_astr(sp->sa, sp->len);
 	    dico_log(L_ERR, errno, ("cannot bind to %s"), p);
 	    free(p);
 	    close(fd);
@@ -128,15 +136,15 @@ open_sockets()
 	}
 
 	if (listen(fd, 8) == -1) {
-	    p = sockaddr_to_astr(&sp->s, socklen);
+	    p = sockaddr_to_astr(sp->sa, sp->len);
 	    dico_log(L_ERR, errno, "cannot listen on %s", p);
 	    free(p);
 	    close(fd);
 	    continue;
 	}
 
-	srvtab[i].addr = &sp->s;
-	srvtab[i].addrlen = socklen;
+	srvtab[i].addr = sp->sa;
+	srvtab[i].addrlen = sp->len;
 	srvtab[i].fd = fd;
 	i++;
 	if (fd > fdmax)
@@ -355,8 +363,8 @@ sig_child(int sig)
 struct sockaddr client_addr;
 socklen_t client_addrlen;
 
-#define ACCESS_DENIED_MSG "530 Access denied\r\n"
-#define TEMP_FAIL_MSG "420 Server temporarily unavailable\r\n"
+#define ACCESS_DENIED_MSG "530 Access denied\n"
+#define TEMP_FAIL_MSG "420 Server temporarily unavailable\n"
 #define SWRITE(fd, s) write(fd, s, sizeof(s)-1)
 
 int
@@ -393,13 +401,12 @@ handle_connection(int n)
 
     if (single_process) {
 	int status;
-	dico_stream_t str;
-
-	str = dico_fd_io_stream_create(connfd, connfd);
-	dico_stream_set_buffer(str, dico_buffer_line, DICO_MAX_BUFFER);
-	status = dicod_loop(str);
-	dico_stream_close(str);
-	dico_stream_destroy(&str);
+	dico_stream_t str = dicod_iostream(connfd, connfd);
+	if (str) {
+	    status = dicod_loop(str);
+	    dico_stream_close(str);
+	    dico_stream_destroy(&str);
+	}
     } else {
 	pid_t pid = fork();
 	if (pid == -1) {
@@ -417,8 +424,9 @@ handle_connection(int n)
 	    signal(SIGCHLD, SIG_DFL);
 	    signal(SIGHUP, SIG_DFL);
         
-	    str = dico_fd_io_stream_create(connfd, connfd);
-	    dico_stream_set_buffer(str, dico_buffer_line, DICO_MAX_BUFFER);
+	    str = dicod_iostream(connfd, connfd);
+	    if (!str)
+		exit(EX_UNAVAILABLE);
 	    status = dicod_loop(str);
 	    dico_stream_close(str);
 	    dico_stream_destroy(&str);
@@ -556,9 +564,14 @@ dicod_server(int argc, char **argv)
     
     dico_log(L_INFO, 0, _("%s started"), program_version);
 
-    if (user_id && switch_to_privs(user_id, group_id, group_list))
-	dico_die(EX_NOUSER, L_CRIT, 0, "exiting");
-
+    if (user_id) {
+	if (getuid())
+	    dico_log(L_NOTICE, 0,
+		     _("not running as root: ignoring user/group settings"));
+	else if (switch_to_privs(user_id, group_id, group_list))
+	    dico_die(EX_NOUSER, L_CRIT, 0, "exiting");
+    }
+    
     if (!foreground)
 	check_pidfile(pidfile_name);
 
@@ -598,7 +611,8 @@ dicod_server(int argc, char **argv)
     if (rc) 
 	dico_log(L_NOTICE, errno, _("Exit code = %d, last error status"), rc);
 
-    remove_pidfile(pidfile_name);
+    if (!foreground)
+	remove_pidfile(pidfile_name);
 
     if (restart) {
 	int i;
