@@ -1,5 +1,5 @@
 /* This file is part of GNU Dico.
-   Copyright (C) 2008, 2010 Sergey Poznyakoff
+   Copyright (C) 2008, 2010, 2012 Sergey Poznyakoff
 
    GNU Dico is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -15,6 +15,10 @@
    along with GNU Dico.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include <gettext.h>
-#define _(String) gettext (String)
-#define N_(String) gettext_noop (String)
+#ifndef _
+# define _(String) gettext (String)
+#endif
+#ifndef N_
+# define N_(String) gettext_noop (String)
+#endif
 
